% generateProblemData.m
%
% Matlab script to generate random matrices P(u) that 
% represent a stochastic shortest path problem. The matrices are stored in
% a three-dimensional array P(i,j,u) of dimension (N+1) x (N+1) x M.
% The (i,j,u)-th element of the array P(i,j,u) denotes the transition 
% probability from node i to the node j when applying the input u.  
% The array P is saved in the file problemData.mat.
%
% Dynamic Programming and Optimal Control
% Fall 2009
% Programming Exercise 2
%
% --
% ETH Zurich
% Institute for Dynamic Systems and Control
% Angela Schllig
% aschoellig@ethz.ch
%
% --
% Revision history
% [10.11.09, AS]    first version
%
%


% clear variable P that will hold the problem data
clear P;


%% Define number of nodes and inputs
% Number of states, NOT including the termination state which is N+1
N = 100;

% Number of control inputs
M = 100;


%% Generate transition probabilities
% Generate the transition probabilities.  There is one for each control
% input.  So for each control input, one can model the transition
% probability as an (N+1) by (N+1) stochastic matrix.  The rows are the starting
% state, the columns the end state.  So the sum of each row must be equal
% to 1.  In addition, we put some structure on the problem: states that are
% "close" to each other have higher probabilities than ones that do not. 
% We use "close" to simply be the numerical difference of the states, with
% wrap-around.  Finally, state N+1 is the termination state.  That means that
% the last row must be equal to [0,...,0,1]: once we get there, we
% stay there.  

% Scaling exponent for transition probabilities.  0 corresponds to all
% states are equally likely, larger exponent makes it more likely that a
% neighbor is the next state.
SCALE_EXPONENT = 2;

% First just generate a whole bunch of entries in the range of 0 to 1.
P = rand(N+1,N+1,M);

% Create scaling matrix.  This is used to scale the probabilities so that
% nearby states, with wrap-around, are more likely.  Faster ways to
% generate this matrix, but this is the most transparent.
scaleMatrix = zeros(N+1,N+1);

for i = 1:N+1
    for j = 1:N+1
        %gives minimum "distance" between nodes (either go forwards or backwards)
        scaleMatrix(i,j) = (min([mod(i-j,N),mod(j-i,N)]));
    end
end

% scale it: farest away = 1, closest = floor((N+1)/2)+1.
scaleMatrix = max(max(scaleMatrix)) - scaleMatrix + 1;
% Accentuate the differences. 
scaleMatrix = scaleMatrix.^SCALE_EXPONENT;

% Can now scale and normalize the transition probabilities.  
for u = 1:M
    % Scale it
    P(:,:,u) = P(:,:,u).*scaleMatrix;
    
    % Normalize it
    for i = 1:N
        P(i,:,u) = P(i,:,u)/sum(P(i,:,u));
    end
    
    % Set the last row appropriately, corresponding to the termination
    % state.
    P(N+1,:,u) = 0*P(N+1,:,u);
    P(N+1,N+1,u) = 1;
end



%% Plot a typical probability transition matrix (the first control)
figure(1)
mesh(P(1:N,1:N,1));
xlabel('Node i');
ylabel('Node j');
zlabel('p_{ij}');
title('Transition Probabilities, u == 1');


%% Save data in mat file
save problemData.mat P;
