% script.m
%
% Matlab script that is used to load the load the problem data, execute the 
% stochasic shortest path algorithms, run the simulation, and display the 
% results.
%
% Dynamic Programming and Optimal Control
% Fall 2009
% Programming Exercise 2
%
% --
% ETH Zurich
% Institute for Dynamic Systems and Control
% Angela Schllig
% aschoellig@ethz.ch
%
% --
% Revision history
% [10.11.09, AS]    first version
%
%


% clear workspace and command window
clear;
clc;


%% Read in problem data
% Load problem matrix P from mat file. P is a three-diomensional array 
% defining the transition probabilities.  
% Uncomment the line of the problem you want to solve.
% load problemData.mat;
load exampleProblemData.mat;   

%% Get problem size
% Number of states, NOT including the termination state which is N+1
N = size(P,1)-1;

% Number of control inputs
M = size(P,3);



%% Do some sanity checks with matrix P:
% Are all entries in P in [0,1]
minVal = min(min(min(P)));
maxVal = max(max(max(P)));
if minVal<0 || maxVal>1
    error('Entries in P are not in [0,1].');
end;

% Are the terminal state conditions satisfied?
if sum(sum(P(N+1,1:N,:)~=0))~=0 || sum(P(N+1,N+1,:)~=1)~=0
    error('Terminal state conditions are not satisfied.');
end;






%% Solution Method 1: Policy Iteration
% Solve the stochastic shortest path (SPP) problem using policy iteration
% and minimize the expected number of transitions from an initial node to 
% the termination state.
%
% Input: problem data P
% Output: optimal cost for any each initial node, optimal policy at each
% node
%
% Use tic, toc to time the execution.
tic;
[policyIterationJ, policyIterationF] = ssp_PolicyIt(P);
policyIterationT = toc;

    




%% Solution Method 2: Value Iteration
% Solve the stochastic shortest path (SPP) problem using value iteration
% and minimize the expected number of transitions from an initial node to
% the termination state. Use the result from method 1 to define a
% meaningful stopping criterion.
%
% Input: problem data P, result from method 1
% Output: optimal cost for any each initial node, optimal policy at each
% node
tic;
[valueIterationJ, valueIterationF] = ssp_ValueIt(P, policyIterationJ);
valueIterationT = toc;






%% Solution Method 3: Linear Programming
% Solve the stochastic shortest path (SPP) problem using linear programming
% and minimize the expected number of transitions from an initial node to
% the termination state. 
%
% Input: problem data P
% Output: optimal cost for any each initial node, optimal policy at each
% node
tic;
[linearProgrammingJ, linearProgrammingF] = ssp_LinearProg(P);
linearProgrammingT = toc;







%% Simulate Problem
% Simulate the system using the optimal policy obtained from method 1 given
% a starting node i0 and a number of simulations SIM_NUM to be performed. 
% Return the simulated costs in vector form. 
%
% Input: optimal policy policyIterationF, starting node i0 and a number of 
% simulations SIM_NUM  
% Output: simulated costs

% Number of simulations to do when simulating optimal strategy.  Want a
% large number to calculate mean and standard deviation.
SIM_NUM = 1e3;

% The starting state is at the center of the wrap-around grid.
i0 = floor(N/2);

simulatedCosts = ssp_Sim (P, i0, SIM_NUM, policyIterationF);

% Calculate mean and standard deviation
avgCost = mean(simulatedCosts);
stdCost = std(simulatedCosts);







%% Display results
% Execution times
disp(' ');
disp('------------------------------------------------------------------');
disp('RESULTS');
disp('Method 1: Policy Iteration');
disp(['  Execution time: ',num2str(policyIterationT),'s.']);
disp(' ');
disp('Method 2: Value Iteration');
disp(['  Execution time: ',num2str(valueIterationT),'s.']);
disp(' ');
disp('Method 3: Linear Programming');
disp(['  Execution time: ',num2str(linearProgrammingT),'s.']);
disp(' ');
disp('------------------------------------------------------------------');
% Simulation Result
disp('SIMULATION');
disp(['Starting node ',num2str(i0)]);
disp(['Simulated average cost ',num2str(avgCost)]);
disp(['Simulated cost standard deviation ',num2str(stdCost)]);
disp(['Expected minimum cost (policy iteration) ',num2str(policyIterationJ(i0))]);
disp(['Expected minimum cost (value iteration) ',num2str(valueIterationJ(i0))]);
disp(['Expected minimum cost (linear programming) ',num2str(linearProgrammingJ(i0))]);
disp(' ');
disp('------------------------------------------------------------------');

% Plot the simulated costs
figure(2)
plot(1:SIM_NUM,simulatedCosts,'.',[1 SIM_NUM],[avgCost avgCost],'k', ...
    [1 SIM_NUM],[avgCost - stdCost, avgCost - stdCost],'r', ...
    [1 SIM_NUM],[avgCost + stdCost, avgCost + stdCost],'r');
xlabel('Simulation #');
ylabel('Cost');
title('Simulated Costs');

% Optimal costs
% The costs calculated via the various methods
figure(3)
plot(1:N,policyIterationJ(1:N),'b.',1:N,valueIterationJ(1:N),'r.',1:N,linearProgrammingJ(1:N),'ko', ...
     [i0],[avgCost],'k*');
legend('Policy Iteration','Value Iteration','Linear Programming','Average Simulated Cost',...
        'Location','South');
title('Cost-to-Go: Value and Policy Iteration, Linear Programming');
xlabel('State');
ylabel('Cost-to-Go');

% Optimal policy from method 1
figure(4)
bar(1:N,policyIterationF(1:N));
title('Optimal Policy from Policy Iteration');
xlabel('State');
ylabel('Optimal Input');
    


    


